"use strict";
// Copyright 2019 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const checkpoint_1 = require("./util/checkpoint");
const github_1 = require("./github");
const parseGithubRepoUrl = require('parse-github-repo-url');
const GITHUB_RELEASE_LABEL = 'autorelease: tagged';
class GitHubRelease {
    constructor(options) {
        this.apiUrl = options.apiUrl;
        this.proxyKey = options.proxyKey;
        this.labels = options.label.split(',');
        this.repoUrl = options.repoUrl;
        this.token = options.token;
        this.packageName = options.packageName;
        this.changelogPath = 'CHANGELOG.md';
        this.gh = this.gitHubInstance(options.octokitAPIs);
    }
    async createRelease() {
        const gitHubReleasePR = await this.gh.findMergedReleasePR(this.labels);
        if (gitHubReleasePR) {
            checkpoint_1.checkpoint(`found release branch ${chalk.green(gitHubReleasePR.version)} at ${chalk.green(gitHubReleasePR.sha)}`, checkpoint_1.CheckpointType.Success);
            const changelogContents = (await this.gh.getFileContents(this.changelogPath)).parsedContent;
            const latestReleaseNotes = GitHubRelease.extractLatestReleaseNotes(changelogContents, gitHubReleasePR.version);
            checkpoint_1.checkpoint(`found release notes: \n---\n${chalk.grey(latestReleaseNotes)}\n---\n`, checkpoint_1.CheckpointType.Success);
            await this.gh.createRelease(this.packageName, gitHubReleasePR.version, gitHubReleasePR.sha, latestReleaseNotes);
            // Add a label indicating that a release has been created on GitHub,
            // but a publication has not yet occurred.
            await this.gh.addLabels([GITHUB_RELEASE_LABEL], gitHubReleasePR.number);
            // Remove 'autorelease: pending' which indicates a GitHub release
            // has not yet been created.
            await this.gh.removeLabels(this.labels, gitHubReleasePR.number);
        }
        else {
            checkpoint_1.checkpoint('no recent release PRs found', checkpoint_1.CheckpointType.Failure);
        }
    }
    gitHubInstance(octokitAPIs) {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({
            token: this.token,
            owner,
            repo,
            apiUrl: this.apiUrl,
            proxyKey: this.proxyKey,
            octokitAPIs,
        });
    }
    static extractLatestReleaseNotes(changelogContents, version) {
        version = version.replace(/^v/, '');
        const latestRe = new RegExp(`## v?\\[?${version}[^\\n]*\\n(.*?)(\\n##\\s|\\n### \\[?[0-9]+\\.|($(?![\r\n])))`, 'ms');
        const match = changelogContents.match(latestRe);
        if (!match) {
            throw Error('could not find changelog entry corresponding to release PR');
        }
        return match[1];
    }
}
exports.GitHubRelease = GitHubRelease;
//# sourceMappingURL=github-release.js.map